import { VFXBatch, RenderMode } from './VFXBatch';
import { BufferGeometry, Layers, Material, Object3D, Texture, Vector3 } from 'three';
import { ParticleEmitter } from './ParticleEmitter';
import { IParticle } from './Particle';
import { FunctionValueGenerator, ValueGenerator } from './functions';
export interface VFXBatchSettings {
    instancingGeometry: BufferGeometry;
    material: Material;
    uTileCount: number;
    vTileCount: number;
    blendTiles: boolean;
    softParticles: boolean;
    softNearFade: number;
    softFarFade: number;
    renderMode: RenderMode;
    renderOrder: number;
    layers: Layers;
}
export interface SerializationOptions {
    useUrlForImage?: boolean;
}
export type RendererEmitterSettings = TrailSettings | MeshSettings | BillBoardSettings | StretchedBillBoardSettings;
export interface StretchedBillBoardSettings {
    speedFactor: number;
    lengthFactor: number;
}
export interface BillBoardSettings {
}
export interface TrailSettings {
    startLength: ValueGenerator | FunctionValueGenerator;
    followLocalOrigin: boolean;
}
export interface MeshSettings {
    rotationAxis?: Vector3;
    startRotationX: ValueGenerator | FunctionValueGenerator;
    startRotationY: ValueGenerator | FunctionValueGenerator;
    startRotationZ: ValueGenerator | FunctionValueGenerator;
}
export interface IParticleSystem {
    worldSpace: boolean;
    particleNum: number;
    duration: number;
    looping: boolean;
    particles: Array<IParticle>;
    emitter: ParticleEmitter<any>;
    _renderer?: BatchedRenderer;
    instancingGeometry: BufferGeometry;
    rendererEmitterSettings: RendererEmitterSettings;
    getRendererSettings(): VFXBatchSettings;
    paused: boolean;
    pause(): void;
    play(): void;
    restart(): void;
    clone(): IParticleSystem;
    toJSON(metaData: any, options: SerializationOptions): any;
}
export declare class BatchedRenderer extends Object3D {
    batches: Array<VFXBatch>;
    systemToBatchIndex: Map<IParticleSystem, number>;
    type: string;
    depthTexture: Texture | null;
    constructor();
    private static equals;
    addSystem(system: IParticleSystem): void;
    deleteSystem(system: IParticleSystem): void;
    setDepthTexture(depthTexture: Texture | null): void;
    updateSystem(system: IParticleSystem): void;
    update(delta: number): void;
}
