import { Matrix4, Quaternion, Vector3, Vector4 } from 'three';
import { EmissionState } from './ParticleSystem';
import { LinkedList } from './util/LinkedList';
export interface IParticle {
    position: Vector3;
    velocity: Vector3;
    age: number;
    life: number;
    size: number;
    rotation?: number | Quaternion;
    uvTile: number;
    color: Vector4;
    get died(): boolean;
}
export interface Particle extends IParticle {
    speedModifier: number;
    emissionState?: EmissionState;
    parentMatrix?: Matrix4;
    startSpeed: number;
    startColor: Vector4;
    startSize: number;
}
export declare class NodeParticle implements IParticle {
    position: Vector3;
    velocity: Vector3;
    age: number;
    life: number;
    size: number;
    angularVelocity?: number | Quaternion;
    rotation: number | Quaternion;
    color: Vector4;
    uvTile: number;
    get died(): boolean;
    reset(): void;
}
export declare class SpriteParticle implements Particle {
    parentMatrix?: Matrix4;
    startSpeed: number;
    startColor: Vector4;
    startSize: number;
    position: Vector3;
    velocity: Vector3;
    age: number;
    life: number;
    size: number;
    speedModifier: number;
    angularVelocity?: number | Quaternion;
    rotation: number | Quaternion;
    color: Vector4;
    uvTile: number;
    get died(): boolean;
}
export declare class RecordState {
    position: Vector3;
    size: number;
    color: Vector4;
    constructor(position: Vector3, size: number, color: Vector4);
}
export declare class TrailParticle implements Particle {
    parentMatrix?: Matrix4;
    startSpeed: number;
    startColor: Vector4;
    startSize: number;
    position: Vector3;
    localPosition?: Vector3;
    velocity: Vector3;
    age: number;
    life: number;
    size: number;
    length: number;
    speedModifier: number;
    color: Vector4;
    previous: LinkedList<RecordState>;
    uvTile: number;
    update(): void;
    get died(): boolean;
    reset(): void;
}
