import { Vector2 } from 'three';
import { NodeValueType } from './NodeValueType';
import { ExecutionContext, NodeDef } from './NodeDef';
export type NodeData = {
    [key: string]: any;
};
export declare class Node {
    id: string;
    inputs: (Wire | ConstInput | Adapter | undefined)[];
    outputs: Wire[][];
    definition: NodeDef;
    signatureIndex: number;
    data: NodeData;
    position: Vector2;
    outputValues: any[];
    constructor(definition: NodeDef, signatureIndex?: number, data?: NodeData);
    get inputTypes(): NodeValueType[];
    get outputTypes(): NodeValueType[];
    func(context: ExecutionContext, inputs: any[], outputs: any[]): void;
}
export interface ConstInput {
    getValue(context: ExecutionContext): any;
}
export declare class Wire {
    input: Node | Adapter;
    inputIndex: number;
    output: Node | Adapter;
    outputIndex: number;
    constructor(input: Node | Adapter, inputIndex: number, output: Node | Adapter, outputIndex: number);
}
export declare class Adapter {
    node: Node;
    isInput: boolean;
    paramIndex: number;
    type: NodeValueType;
    inputs: (Wire | undefined)[];
    outputs: Wire[][];
    constructor(type: NodeValueType, isInput: boolean, node: Node, paramIndex: number);
}
