import { IParticle } from '../Particle';
import { ParticleEmitter } from '../ParticleEmitter';
import { Object3DEventMap, BufferGeometry, Layers, Material, Matrix4, Texture } from 'three';
import { RenderMode } from '../VFXBatch';
import { BatchedRenderer, IParticleSystem, SerializationOptions, VFXBatchSettings } from '../BatchedRenderer';
import { NodeGraph } from './NodeGraph';
import { Interpreter } from './Interpreter';
import { BillBoardSettings, MeshSettings, TrailSettings } from '../BatchedRenderer';
export interface VFXParameters {
    autoDestroy?: boolean;
    looping?: boolean;
    prewarm?: boolean;
    duration?: number;
    emissionGraph: NodeGraph;
    updateGraph: NodeGraph;
    instancingGeometry?: BufferGeometry;
    renderMode?: RenderMode;
    rendererEmitterSettings?: TrailSettings | MeshSettings | BillBoardSettings;
    speedFactor?: number;
    material: Material;
    layers?: Layers;
    renderOrder?: number;
    worldSpace?: boolean;
}
interface EmissionState {
    time: number;
}
export declare class NodeVFX implements IParticleSystem {
    emissionGraph: NodeGraph;
    updateGraph: NodeGraph;
    interpreter: Interpreter;
    autoDestroy: boolean;
    prewarm: boolean;
    looping: boolean;
    duration: number;
    particleNum: number;
    paused: boolean;
    particles: Array<IParticle>;
    emitter: ParticleEmitter<Object3DEventMap>;
    rendererSettings: VFXBatchSettings;
    neededToUpdateRender: boolean;
    rendererEmitterSettings: {};
    worldSpace: boolean;
    private prewarmed;
    private emissionState;
    private emitEnded;
    private markForDestroy;
    private previousWorldPos?;
    private temp;
    private travelDistance;
    private normalMatrix;
    _renderer?: BatchedRenderer;
    set time(time: number);
    get time(): number;
    get layers(): Layers;
    get texture(): Texture;
    set texture(texture: Texture);
    get material(): Material;
    set material(material: Material);
    get instancingGeometry(): BufferGeometry;
    set instancingGeometry(geometry: BufferGeometry);
    get renderMode(): RenderMode;
    set renderMode(renderMode: RenderMode);
    get renderOrder(): number;
    set renderOrder(renderOrder: number);
    get blending(): import("three").Blending;
    set blending(blending: import("three").Blending);
    constructor(parameters: VFXParameters);
    pause(): void;
    play(): void;
    private spawn;
    endEmit(): void;
    dispose(): void;
    restart(): void;
    private update;
    emit(delta: number, emissionState: EmissionState, emitterMatrix: Matrix4): void;
    toJSON(meta: any, options?: SerializationOptions): any;
    getRendererSettings(): VFXBatchSettings;
    clone(): IParticleSystem;
    speedFactor: number;
}
export {};
