declare const _default: "\n////////////////////////////////////////////////////////////////////////////////\n// Vertex shader\n////////////////////////////////////////////////////////////////////////////////\nstruct RenderParams {\n  modelViewProjectionMatrix : mat4x4f,\n  right : vec3<f32>,\n  up : vec3<f32>\n}\n\nstruct Particle {\n    color: vec4<f32>,\n    position: vec3<f32>,\n    velocity: vec3<f32>,\n    life: f32,\n    age: f32,\n}\n\n@group(0) @binding(0) var<uniform> render_params : RenderParams;\n@group(0) @binding(1) var<storage, read> particles: array<Particle>;\n@group(0) @binding(2) var<storage, read> particleIndices: array<u32>;\n\nstruct VertexInput {\n  /*@location(0) position : vec3f,\n  @location(1) color : vec4f,*/\n  @location(0) quad_pos : vec2f, // -1..+1\n  @builtin(instance_index) instanceIndex: u32,\n}\n\nstruct VertexOutput {\n  @builtin(position) position : vec4f,\n  @location(0) color : vec4f,\n  @location(1) quad_pos : vec2f, // -1..+1\n}\n\n@vertex\nfn vs_main(in : VertexInput) -> VertexOutput {\n  var quad_pos = mat2x3f(render_params.right, render_params.up) * in.quad_pos;\n  \n  var particle = particles[particleIndices[in.instanceIndex]];\n  var position = particle.position + quad_pos * 0.01;\n  var out : VertexOutput;\n  out.position = render_params.modelViewProjectionMatrix * vec4f(position, 1.0);\n  out.color = particle.color;\n  out.quad_pos = in.quad_pos;\n  return out;\n}\n\n////////////////////////////////////////////////////////////////////////////////\n// Fragment shader\n////////////////////////////////////////////////////////////////////////////////\n@fragment\nfn fs_main(in : VertexOutput) -> @location(0) vec4f {\n  var color = vec4<f32>(1.0, 0.0, 0.0, 1.0); //in.color;\n  // Apply a circular particle alpha mask\n  color.a = color.a * max(1.0 - length(in.quad_pos), 0.0);\n  return color;\n}\n\n";
export default _default;
